#ifndef GPIB_H_
#define GPIB_H_
#include "integer.h"
#include <stdint.h>
#include "lifdisk.h"
#include "amigo.h"
//
// GPIB defines
//
// (c) 2010 - 2019 Anders Gustafsson <anders.gustafsson@pedago.fi> 
//
// From hpdisk2:
//
// Moved SRQ from A2 to A7
//
// PIC Pin addignments for original HPDisk GPIB PCB
//
// RA0 - Parallell Poll status bit
// RA1 - Listening LED/FLAG, activer high
// RA2 - Talking LED/FLAG
// RA3 -
// RA4 -
// RA5 -
// RA6 - XTAL, not used with 46K40, trigger 1
// RA7 - XTAL, -"-, trigger 2
//
// Control/status Port
// RB0 - DAV
// RB1 - NFRD
// RB2 - NDAC
// RB3 - REN
// RB4 - ATN
// RB5 - EOI
// RB6 - IFC
// RB7 - SRQ
//
// RC0 -
// RC1 -
// RC2 -
// RC3 -
// RC4 -
// RC5 -
// RC6 - TX
// RC7 - RX
//
// DATA Port
// RD0 - D1
// RD1 - D2
// RD2 - D3
// RD3 - D4
// RD4 - D5
// RD5 - D6
// RD6 - D7
// RD7 - D8
//
// RE0 - A0 address select for LIF file
// RE1 - A1 -"-
// RE2 - A2 -"-
//
// GPIB Connector pinout
//
//Pin  	Name 	Signal Description 			Pin Name 	Signal Description 
//1 	DIO1 	Data Input/Output Bit 1 	13 	DIO5 	Data Input/Output Bit 5 
//2 	DIO2 	Data Input/Output Bit 2 	14 	DIO6 	Data Input/Output Bit 6 
//3 	DIO3 	Data Input/Output Bit 3 	15 	DIO7 	Data Input/Output Bit 7 
//4 	DIO4 	Data Input/Output Bit 4 	16 	DIO8 	Data Input/Output Bit 8 
//5 	EIO 	End-Or-Identify 			17 	REN 	Remote Enable 
//6 	DAV 	Data Valid 					18 	Shield 	Ground (DAV) 
//7 	NRFD 	Not Ready For Data 			19 	Shield 	Ground (NRFD) 
//8 	NDAC 	Not Data Accepted 			20 	Shield 	Ground (NDAC) 
//9 	IFC 	Interface Clear 			21 	Shield 	Ground (IFC) 
//10 	SRQ 	Service Request 			22 	Shield 	Ground (SRQ) 
//11 	ATN 	Attention 					23 	Shield 	Ground (ATN) 
//12 	Shield 	Chassis Ground 				24 	Single GND 	Single Ground
//

#define DIO_PORT	PORTD
#define	DIO_LAT		LATD
#define	DIO_TRIS	TRISD

#define STATUS_PORT	PORTB
#define	STATUS_LAT	LATB
#define	STATUS_TRIS	TRISB

#define DAV_PORT	PORTBbits.RB0
#define DAV_LAT		LATBbits.LATB0
#define DAV_TRIS	TRISBbits.TRISB0

#define NRFD_PORT	PORTBbits.RB1
#define NRFD_LAT	LATBbits.LATB1
#define NRFD_TRIS	TRISBbits.TRISB1

#define NDAC_PORT	PORTBbits.RB2
#define NDAC_LAT	LATBbits.LATB2
#define NDAC_TRIS	TRISBbits.TRISB2

#define REN_PORT	PORTBbits.RB3
#define REN_LAT		LATBbits.LATB3
#define REN_TRIS    TRISBbits.TRISB3
#define REN_BIT     0b00001000

#define ATN_PORT	PORTBbits.RB4
#define ATN_LAT		LATBbits.LATB4
#define ATN_TRIS	TRISBbits.TRISB4
#define ATN_BIT     0b00010000          // Used to check ATN in the status byte

#define EOI_PORT	PORTBbits.RB5
#define EOI_LAT		LATBbits.LATB5
#define EOI_TRIS    TRISBbits.TRISB5
#define EOI_BIT     0b00100000

#define IFC_PORT	PORTBbits.RB6
#define IFC_LAT		LATBbits.LATB6
#define IFC_TRIS	TRISBbits.TRISB6
#define IFC_BIT     0b01000000

#define SRQ_PORT	PORTBbits.RB7
#define SRQ_LAT		LATBbits.LATB7
#define SRQ_TRIS	TRISBbits.TRISB7
#define SRQ_BIT     0b10000000

#define PP_LATCH	LATAbits.LATA0		// LATCH used to set response to parallell poll. 0=active, 1=inactive
#define PP_PORT     PORTAbits.RA0

#define listening   LATAbits.LATA1
#define talking     LATAbits.LATA2
//
#define MAXCMDLEN 64
#define FLAGS_AUTO	1			// True if instrument sets to talk after command
#define FLAGS_EOI	2			// True in case we should set EOI on last char of command
#define FLAGS_EOT_ENABLE	4	// True in case we should add an EOT char after EOI
#define FLAGS_MODE	8			// True if we are controller
#define FLAGS_LON	16			// True if we are in listen only mode
//

#define GPIB_S_DAT	1
#define GPIB_R_DAT	2
#define GPIB_R_SRQ	3
#define RD_AD_CMD 	4
#define SET_IO_CMD 	5
#define RESET_CMD       6

#define FLASH_SECTION_READ     0x01
#define FLASH_SECTION_WRITE    0x02

#define TIMEOUT 10000
#define true 1
#define false 0
//
// GPIB Commands
//
//#define MLA		0x25		// My listen address = 0 (0+0x20) (for disk 0)
//#define	MTA		0x45		// My talk address = 0 (0+0x40)
//#define	UNL		0x3F		// Unlisten
//#define UNT		0x5F		// Untalk
//#define	LLO		0x11		// Local lockout
//#define	DCL		0x14		// Device clear
//#define	PPU		0x15		// Parallell poll unconfigure
//#define SPE		0x18		// Serial poll enable
//#define	SPD		0x19		// Serial poll disable
//#define	GTL		0x01		// Go to local
//#define SDC		0x04		// Selected device clear
//#define PPC		0x05		// Parallell poll configure
//#define GET		0x08		// Group execute trigger
//#define TCT		0x09		// Take control
//
// GPIB debug bits
//
#define GPIB_DBG_FROM_USB	1
#define GPIB_DBG_FROM_GPIB	2
#define GPIB_DBG_PROLOGIX	4
#define GPIB_DBG_TX			8
#define GPIB_DBG_RX			16
#define GPIB_DBG_CMD		32
#define GPIB_DBG_SPOLL		64
#define GPIB_DBG_GPIB_CMD	128
//
// GPIB receive finite-state-machine states
//
#define GPIB_RX_START			0
#define GPIB_RX_ACCEPT			1
#define GPIB_RX_WAIT_DAV 		2
#define GPIB_RX_DAV_LOW			3
#define GPIB_RX_WAIT_DAV_HIGH 	4
#define GPIB_RX_DAV_HIGH 		5
#define GPIB_RX_EOI				6
#define GPIB_RX_FINISH 			9
#define GPIB_RX_DAV_TIMEOUT		10
#define GPIB_RX_DONE 			99

//
// GPIB send finite-state-machine states
//
#define GPIB_TX_START			0
#define GPIB_TX_CHECK			1
#define GPIB_TX_PUT_DATA		2
#define GPIB_TX_WAIT_FOR_NRFD	3
#define GPIB_TX_SET_DAV_LOW		4
#define GPIB_TX_WAIT_FOR_NDAC	5
#define GPIB_TX_SET_DAV_HIGH	6
#define GPIB_TX_FINISH 			9
#define GPIB_TX_ERROR 			98
#define GPIB_TX_DONE 			99

#define GPIB_MAX_TX_LEN			256		// Max length of transmit GPIB string
#define GPIB_MAX_RX_LEN			256		// Max length of receive GPIB string

extern char gpib_rx_buff[GPIB_MAX_RX_LEN];
extern char gpib_ptr;
extern char gpib_tx_buff[512];//GPIB_MAX_TX_LEN];
extern unsigned int gpib_tx_ptr;
//extern unsigned near char listening;
//extern unsigned near char talking;

//typedef union {
//    struct {
//        unsigned status0                 :1;    // 0..4 are used by adress switch
//        unsigned status1                 :1;
//        unsigned status2                 :1;
//        unsigned status3                 :1;
//        unsigned status4                 :1;
//        unsigned status5                 :1;
//        unsigned listening                 :1;
//        unsigned talking                 :1;
//    };
//    unsigned char status;
//} GPIBStatus_t;
//
//extern GPIBStatus_t GPIBStatus;

extern unsigned char mla;
extern unsigned char mta;
extern unsigned char spoll;
extern unsigned char spbyte;

extern unsigned char gpib_flags;
extern char gpib_eot_char;
extern unsigned char gpib_addr;

extern unsigned char eoiflag;
extern unsigned char atnflag;
extern unsigned char can_break;

int gpib_test (void);
BYTE gpib_getc (char *cbyte);
//

void gpib_init(void);
int sendgpib (int len);

void gpib_srq(unsigned char state);

void gpib_cmd(unsigned char ch);
void dump(void);
void DumpData(char *Ptr,int length, char *label);
int gpib_flush(void);
void trigger(char n);

#if defined(SOFTWARE_PP)
#warning Compiled for SOFTWARE PP
#define EnablePP()	PP_BYTE = 0x7f;PP_LATCH = 1;	// Software Parallell Poll response
#define DisablePP()	PP_BYTE = 0xff;PP_LATCH = 1;
#else
//#warning Compiled for HARDWARE PP
#define EnablePP()	PP_LATCH = 0;	// Hardware Parallell Poll response
#define DisablePP()	PP_LATCH = 1;
#endif

#endif
